import http2Client from 'http2-client'

export async function findResponsiveUrl ({ urls, path, headers = {}, secure } = {}) {
  return Promise.any(urls.map(url => new Promise((resolve, reject) => {
    const { hostname: host, port, protocol } = url
    const req = http2Client.request({ host, port, protocol, path, headers, rejectUnauthorized: secure }, (res) => {
      if (res.statusCode >= 400) reject(new Error(`Error: ${res.statusCode} ${res.message}`))
      resolve(url)
    })
    req.end()
  })))
}

export async function pipe (proxyRes, res, opt = {}) {
  return new Promise((resolve, reject) => {
    opt.bytesWritten = opt.bytesWritten || 0

    // TODO proper error handling and forwarding
    if (proxyRes.statusCode >= 400) {
      reject(new Error(`Error: ${proxyRes.statusCode} ${proxyRes.message}`))
    }

    function setHeaders () {
      if (!opt.bytesWritten) {
        res.statusCode = proxyRes.statusCode
        for (const [key, value] of Object.entries(proxyRes.headers)) {
          res.setHeader(key, value)
        }
      }
    }

    function onDrain () {
      proxyRes.resume()
    }

    // pipe the response data from proxyRes to res
    proxyRes
      .on('data', buf => {
        setHeaders()
        opt.bytesWritten += buf.length
        if (!res.write(buf)) {
          proxyRes.pause()
        }
      })
      .on('end', () => {
        setHeaders()
        // trailers are only there, when version is compliant
        // proxyRes.trailers contains headers that are sent at the end of the request
        if (proxyRes.trailers) res.addTrailers(proxyRes.trailers)
        res.end()
        resolve()
      })
      .on('close', () => {
        res.off('drain', onDrain)
      })

    res.on('drain', onDrain)
  })
}
