import { PROJECT_NAME as name } from './constants.js'
import manifestMiddleware from './middlewares/manifest.js'
import proxyMiddleware from './middlewares/proxy.js'
import upgradeMiddleware from './middlewares/upgrade.js'

export default function (config) {
  return {
    name,

    async configureServer ({ config: { server: serverConfig }, middlewares, httpServer }) {
      const proxies = Object.entries(config.proxy).map(([condition, config]) => {
        // prepare proxy config
        const { hostname, protocol, port } = new URL(config.target)
        // TODO convert conditions starting with ^ into regex
        return [condition, {
          ...config,
          hostname,
          protocol,
          port
        }]
      })
      const webProxies = proxies.filter(([c, { ws }]) => ws !== true)
      const wsProxies = proxies.filter(([c, { ws }]) => ws === true)
      // TODO check what happens if the same port is already covered
      const protocol = serverConfig.https ? 'https:' : 'http:'
      const hostname = serverConfig.host || '127.0.0.1'
      const port = serverConfig.port ? `:${serverConfig.port}` : ''
      const frontendUrls = [`${protocol}//${hostname}${port}`].concat(config.frontendURIs).filter(Boolean).map(uri => new URL(uri))

      middlewares.use(manifestMiddleware({
        frontendUrls
      }))

      middlewares.use(proxyMiddleware({
        proxyURLs: webProxies,
        frontendUrls
      }))

      httpServer.on('upgrade', upgradeMiddleware({
        proxyURLs: wsProxies
      }))
    }
  }
}
