import parseurl from 'parseurl'
import * as proxy from '../proxy.js'

function mergeManifests (manifests) {
  const manifestsByHash = {}

  manifests.forEach(manifest => {
    const hash = `${manifest.namespace}--${manifest.path}`
    // overwrites existing ones. last wins
    manifestsByHash[hash] = manifest
  })
  return Object.values(manifestsByHash)
}

export default function ({ frontendUrls }) {
  return async function (req, res, next) {
    if (req.headers['proxied-by-ox-proxy']) return next()
    const { path } = parseurl(req)
    if (path !== '/api/manifest.json') return next()

    const responses = await proxy.all(req, res, { urls: frontendUrls, headers: { 'proxied-by-ox-proxy': true } })
    const data = mergeManifests(responses.flat())

    // merge all manifests
    res.statusCode = 200
    res.setHeader('Content-Type', 'application/json')
    res.write(JSON.stringify(data))
    return res.end()
  }
}
