import proxy from 'http2-proxy'
import { pipe } from './http.js'

export async function all (req, res, { urls, headers: additionalHeaders }) {
  if (req.aborted || res.readableEnded) return

  return Promise.all(urls.map(async ({ hostname, protocol, port }) => {
    let buffer = ''
    let headers
    await new Promise((resolve, reject) => proxy.web(req, res, {
      hostname,
      protocol,
      port,
      onReq (req, { headers }) {
        Object.assign(headers, additionalHeaders)
      },
      onRes (req, res, proxyRes) {
        // TODO proper error handling and forwarding
        headers = proxyRes.headers
        proxyRes
          .on('data', buf => (buffer += buf.toString()))
          .on('end', () => resolve())
      }
    }))
    if (headers['content-type'] === 'application/json') {
      return JSON.parse(buffer)
    }
    return buffer
  }))
}

export async function web (req, res, { hostname, protocol, port, headers: additionalHeaders }) {
  if (req.aborted || res.readableEnded) return

  const opt = {}
  return proxy.web(req, res, {
    hostname,
    protocol,
    port,
    onReq (req, { headers }) {
      Object.assign(headers, additionalHeaders)
    },
    onRes (req, res, proxyRes) {
      return pipe(proxyRes, res, opt)
    }
  })
}

export async function ws (req, socket, head, { hostname, protocol, port }) {
  return proxy.ws(req, socket, head, {
    hostname,
    protocol,
    port
  })
}
